#include <math.h>

#include "bstream.h"
#include "sbt.h"
#include "mpegdecode.h"

extern MPEG_HEADER		gMpegHeader;
extern DECODE_OPTION	gDecOption;
extern DECODE_INFO		gDecInfo;

extern int nsb_limit;
extern float sample[2304];

/* by init */
static float look_c_valueL2[18];
static signed char group3_table[32][3];
static signed char group5_table[128][3];
static signed short group9_table[1024][3];

/* Layer 1,2 global */
int max_sb;
int stereo_sb;

const int look_joint[16] =
{				/* lookup stereo sb's by mode+ext */
   64, 64, 64, 64,		/* stereo */
   2 * 4, 2 * 8, 2 * 12, 2 * 16,	/* joint */
   64, 64, 64, 64,		/* dual */
   32, 32, 32, 32,		/* mono */
};

SBT_FUNCTION sbt = (SBT_FUNCTION )sbt_mono;
SBT_FUNCTION sbt_table[2][3][5] =
{
    {
        {
	(SBT_FUNCTION) sbt_mono,
	(SBT_FUNCTION) sbt_dual,
	(SBT_FUNCTION) sbt_dual_mono,
	(SBT_FUNCTION) sbt_dual_left,
	(SBT_FUNCTION) sbt_dual_right,
        },{
	(SBT_FUNCTION) sbt16_mono,
	(SBT_FUNCTION) sbt16_dual,
	(SBT_FUNCTION) sbt16_dual_mono,
	(SBT_FUNCTION) sbt16_dual_left,
	(SBT_FUNCTION) sbt16_dual_right,
        },{
	(SBT_FUNCTION) sbt8_mono,
	(SBT_FUNCTION) sbt8_dual,
	(SBT_FUNCTION) sbt8_dual_mono,
	(SBT_FUNCTION) sbt8_dual_left,
	(SBT_FUNCTION) sbt8_dual_right,
        }
    },{
        {
	(SBT_FUNCTION) sbtB_mono,
	(SBT_FUNCTION) sbtB_dual,
	(SBT_FUNCTION) sbtB_dual_mono,
	(SBT_FUNCTION) sbtB_dual_left,
	(SBT_FUNCTION) sbtB_dual_right,
        },{
	(SBT_FUNCTION) sbtB16_mono,
	(SBT_FUNCTION) sbtB16_dual,
	(SBT_FUNCTION) sbtB16_dual_mono,
	(SBT_FUNCTION) sbtB16_dual_left,
	(SBT_FUNCTION) sbtB16_dual_right,
        },{
	(SBT_FUNCTION) sbtB8_mono,
	(SBT_FUNCTION) sbtB8_dual,
	(SBT_FUNCTION) sbtB8_dual_mono,
	(SBT_FUNCTION) sbtB8_dual_left,
	(SBT_FUNCTION) sbtB8_dual_right,
        }
    }
};

/* work Layer1,2 */
int bat[4][16];	/* set by unpack_ba */
int ballo[64];	/* set by unpack_ba */
unsigned int samp_dispatch[66];	/* set by unpack_ba */
float c_value[64];	/* set by unpack_ba */

unsigned int sf_dispatch[66];	/* set by unpack_ba */
float sf_table[64];
float cs_factor[3][64];

static int bit_skip;
static const int bat_bit_masterL2[] =
{
   0, 5, 7, 9, 10, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48
};

static int nbat[4] = {3, 8, 12, 7};

/* ABCD_INDEX = lookqt[mode][sr_index][br_index]  */
/* -1 = invalid  */
static const char lookqt[4][3][16] =
{
    {
        {
   1, -1, -1, -1, 2, -1, 2, 0, 0, 0, 1, 1, 1, 1, 1, -1		/*  44ks stereo */
        },{
   0, -1, -1, -1, 2, -1, 2, 0, 0, 0, 0, 0, 0, 0, 0, -1		/*  48ks */
        },{
   1, -1, -1, -1, 3, -1, 3, 0, 0, 0, 1, 1, 1, 1, 1, -1		/*  32ks */
        }
    },{
        {
   1, -1, -1, -1, 2, -1, 2, 0, 0, 0, 1, 1, 1, 1, 1, -1		/*  44ks joint stereo */
        },{
   0, -1, -1, -1, 2, -1, 2, 0, 0, 0, 0, 0, 0, 0, 0, -1		/*  48ks */
        },{
   1, -1, -1, -1, 3, -1, 3, 0, 0, 0, 1, 1, 1, 1, 1, -1		/*  32ks */
        }
    },{
        {
   1, -1, -1, -1, 2, -1, 2, 0, 0, 0, 1, 1, 1, 1, 1, -1		/*  44ks dual chan */
        },{
   0, -1, -1, -1, 2, -1, 2, 0, 0, 0, 0, 0, 0, 0, 0, -1		/*  48ks */
        },{
   1, -1, -1, -1, 3, -1, 3, 0, 0, 0, 1, 1, 1, 1, 1, -1		/*  32ks */
        }
    },{
// mono extended beyond legal br index
//  1,2,2,0,0,0,1,1,1,1,1,1,1,1,1,-1,          /*  44ks single chan */
//  0,2,2,0,0,0,0,0,0,0,0,0,0,0,0,-1,          /*  48ks */
//  1,3,3,0,0,0,1,1,1,1,1,1,1,1,1,-1,          /*  32ks */
// legal mono
        {
   1, 2, 2, 0, 0, 0, 1, 1, 1, 1, 1, -1, -1, -1, -1, -1		/*  44ks single chan */
        },{
   0, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, -1, -1, -1, -1, -1		/*  48ks */
        },{
   1, 3, 3, 0, 0, 0, 1, 1, 1, 1, 1, -1, -1, -1, -1, -1		/*  32ks */
        }
    }
};

/* bit allocation table look up */
/* table per mpeg spec tables 3b2a/b/c/d  /e is mpeg2 */
/* look_bat[abcd_index][4][16]  */
static const unsigned char look_bat[5][4][16] =
{
/* LOOK_BATA */
    {
   {0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17 },
   {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 17 },
   {0, 1, 2, 3, 4, 5, 6, 17, 0, 0, 0, 0, 0, 0, 0, 0 },
   {0, 1, 2, 17, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
/* LOOK_BATB */
    },{
   {0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17 },
   {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 17 },
   {0, 1, 2, 3, 4, 5, 6, 17, 0, 0, 0, 0, 0, 0, 0, 0 },
   {0, 1, 2, 17, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
/* LOOK_BATC */
    },{
   {0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 },
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
   {0, 1, 2, 4, 5, 6, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0 },
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
/* LOOK_BATD */
    },{
   {0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16},
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
   {0, 1, 2, 4, 5, 6, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0},
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
/* LOOK_BATE */
    },{
   {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
   {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
   {0, 1, 2, 4, 5, 6, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0},
   {0, 1, 2, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    }
};

/* look_nbat[abcd_index]][4] */
static const unsigned char look_nbat[5][4] =
{
   { 3, 8, 12, 4 },
   { 3, 8, 12, 7 },
   { 2, 0, 6, 0 },
   { 2, 0, 10, 0 },
   { 4, 0, 7, 19 }
};

static void unpack_ba()
{
   int i, j, k;
   static int nbit[4] =
   {4, 4, 3, 2};
   int nstereo;

   bit_skip = 0;
   nstereo = stereo_sb;
   k = 0;
   for (i = 0; i < 4; i++)
   {
      for (j = 0; j < nbat[i]; j++, k++)
      {
	 bitget_check(4);
	 ballo[k] = samp_dispatch[k] = bat[i][mac_bitget(nbit[i])];
	 if (k >= nsb_limit)
	    bit_skip += bat_bit_masterL2[samp_dispatch[k]];
	 c_value[k] = look_c_valueL2[samp_dispatch[k]];
	 if (--nstereo < 0)
	 {
	    ballo[k + 1] = ballo[k];
	    samp_dispatch[k] += 18;	/* flag as joint */
	    samp_dispatch[k + 1] = samp_dispatch[k];	/* flag for sf */
	    c_value[k + 1] = c_value[k];
	    k++;
	    j++;
	 }
      }
   }
   samp_dispatch[nsb_limit] = 37;	/* terminate the dispatcher with skip */
   samp_dispatch[k] = 36;	/* terminate the dispatcher */

}
/*-------------------------------------------------------------------------*/
static void unpack_sfs()	/* unpack scale factor selectors */
{
   int i;

   for (i = 0; i < max_sb; i++)
   {
      bitget_check(2);
      if (ballo[i])
	 sf_dispatch[i] = mac_bitget(2);
      else
	 sf_dispatch[i] = 4;	/* no allo */
   }
   sf_dispatch[i] = 5;		/* terminate dispatcher */
}
/*-------------------------------------------------------------------------*/
static void unpack_sf()		/* unpack scale factor */
{				/* combine dequant and scale factors */
   int i;

   i = -1;
 dispatch:switch (sf_dispatch[++i])
   {
      case 0:			/* 3 factors 012 */
	 bitget_check(18);
	 cs_factor[0][i] = c_value[i] * sf_table[mac_bitget(6)];
	 cs_factor[1][i] = c_value[i] * sf_table[mac_bitget(6)];
	 cs_factor[2][i] = c_value[i] * sf_table[mac_bitget(6)];
	 goto dispatch;
      case 1:			/* 2 factors 002 */
	 bitget_check(12);
	 cs_factor[1][i] = cs_factor[0][i] = c_value[i] * sf_table[mac_bitget(6)];
	 cs_factor[2][i] = c_value[i] * sf_table[mac_bitget(6)];
	 goto dispatch;
      case 2:			/* 1 factor 000 */
	 bitget_check(6);
	 cs_factor[2][i] = cs_factor[1][i] = cs_factor[0][i] =
	    c_value[i] * sf_table[mac_bitget(6)];
	 goto dispatch;
      case 3:			/* 2 factors 022 */
	 bitget_check(12);
	 cs_factor[0][i] = c_value[i] * sf_table[mac_bitget(6)];
	 cs_factor[2][i] = cs_factor[1][i] = c_value[i] * sf_table[mac_bitget(6)];
	 goto dispatch;
      case 4:			/* no allo */
/*-- cs_factor[2][i] = cs_factor[1][i] = cs_factor[0][i] = 0.0;  --*/
	 goto dispatch;
      case 5:			/* all done */
	 ;
   }				/* end switch */
}
/*-------------------------------------------------------------------------*/
#define UNPACK_N(n) s[k]     =  cs_factor[i][k]*(bitget(n)-((1 << (n-1)) -1));   \
    s[k+64]  =  cs_factor[i][k]*(bitget(n)-((1 << (n-1)) -1));   \
    s[k+128] =  cs_factor[i][k]*(bitget(n)-((1 << (n-1)) -1));   \
    goto dispatch;
#define UNPACK_N2(n) bitget_check(3*n);                                         \
    s[k]     =  cs_factor[i][k]*(mac_bitget(n)-((1 << (n-1)) -1));   \
    s[k+64]  =  cs_factor[i][k]*(mac_bitget(n)-((1 << (n-1)) -1));   \
    s[k+128] =  cs_factor[i][k]*(mac_bitget(n)-((1 << (n-1)) -1));   \
    goto dispatch;
#define UNPACK_N3(n) bitget_check(2*n);                                         \
    s[k]     =  cs_factor[i][k]*(mac_bitget(n)-((1 << (n-1)) -1));   \
    s[k+64]  =  cs_factor[i][k]*(mac_bitget(n)-((1 << (n-1)) -1));   \
    bitget_check(n);                                           \
    s[k+128] =  cs_factor[i][k]*(mac_bitget(n)-((1 << (n-1)) -1));   \
    goto dispatch;
#define UNPACKJ_N(n) tmp        =  (bitget(n)-((1 << (n-1)) -1));                 \
    s[k]       =  cs_factor[i][k]*tmp;                       \
    s[k+1]     =  cs_factor[i][k+1]*tmp;                     \
    tmp        =  (bitget(n)-((1 << (n-1)) -1));                 \
    s[k+64]    =  cs_factor[i][k]*tmp;                       \
    s[k+64+1]  =  cs_factor[i][k+1]*tmp;                     \
    tmp        =  (bitget(n)-((1 << (n-1)) -1));                 \
    s[k+128]   =  cs_factor[i][k]*tmp;                       \
    s[k+128+1] =  cs_factor[i][k+1]*tmp;                     \
    k++;       /* skip right chan dispatch */                \
    goto dispatch;
/*-------------------------------------------------------------------------*/
static void unpack_samp()	/* unpack samples */
{
   int i, j, k;
   float *s;
   int n;
   long tmp;

   s = sample;
   for (i = 0; i < 3; i++)
   {				/* 3 groups of scale factors */
      for (j = 0; j < 4; j++)
      {
	 k = -1;
       dispatch:switch (samp_dispatch[++k])
	 {
	    case 0:
	       s[k + 128] = s[k + 64] = s[k] = 0.0F;
	       goto dispatch;
	    case 1:		/* 3 levels grouped 5 bits */
	       bitget_check(5);
	       n = mac_bitget(5);
	       s[k] = cs_factor[i][k] * group3_table[n][0];
	       s[k + 64] = cs_factor[i][k] * group3_table[n][1];
	       s[k + 128] = cs_factor[i][k] * group3_table[n][2];
	       goto dispatch;
	    case 2:		/* 5 levels grouped 7 bits */
	       bitget_check(7);
	       n = mac_bitget(7);
	       s[k] = cs_factor[i][k] * group5_table[n][0];
	       s[k + 64] = cs_factor[i][k] * group5_table[n][1];
	       s[k + 128] = cs_factor[i][k] * group5_table[n][2];
	       goto dispatch;
	    case 3:
	       UNPACK_N2(3)	/* 7 levels */
	    case 4:		/* 9 levels grouped 10 bits */
	       bitget_check(10);
	       n = mac_bitget(10);
	       s[k] = cs_factor[i][k] * group9_table[n][0];
	       s[k + 64] = cs_factor[i][k] * group9_table[n][1];
	       s[k + 128] = cs_factor[i][k] * group9_table[n][2];
	       goto dispatch;
	    case 5:
	       UNPACK_N2(4)	/* 15 levels */
	    case 6:
	       UNPACK_N2(5)	/* 31 levels */
	    case 7:
	       UNPACK_N2(6)	/* 63 levels */
	    case 8:
	       UNPACK_N2(7)	/* 127 levels */
	    case 9:
	       UNPACK_N2(8)	/* 255 levels */
	    case 10:
	       UNPACK_N3(9)	/* 511 levels */
	    case 11:
	       UNPACK_N3(10)	/* 1023 levels */
	    case 12:
	       UNPACK_N3(11)	/* 2047 levels */
	    case 13:
	       UNPACK_N3(12)	/* 4095 levels */
	    case 14:
	       UNPACK_N(13)	/* 8191 levels */
	    case 15:
	       UNPACK_N(14)	/* 16383 levels */
	    case 16:
	       UNPACK_N(15)	/* 32767 levels */
	    case 17:
	       UNPACK_N(16)	/* 65535 levels */
/* -- joint ---- */
	    case 18 + 0:
	       s[k + 128 + 1] = s[k + 128] = s[k + 64 + 1] = s[k + 64] = s[k + 1] = s[k] = 0.0F;
	       k++;		/* skip right chan dispatch */
	       goto dispatch;
	    case 18 + 1:	/* 3 levels grouped 5 bits */
	       n = bitget(5);
	       s[k] = cs_factor[i][k] * group3_table[n][0];
	       s[k + 1] = cs_factor[i][k + 1] * group3_table[n][0];
	       s[k + 64] = cs_factor[i][k] * group3_table[n][1];
	       s[k + 64 + 1] = cs_factor[i][k + 1] * group3_table[n][1];
	       s[k + 128] = cs_factor[i][k] * group3_table[n][2];
	       s[k + 128 + 1] = cs_factor[i][k + 1] * group3_table[n][2];
	       k++;		/* skip right chan dispatch */
	       goto dispatch;
	    case 18 + 2:	/* 5 levels grouped 7 bits */
	       n = bitget(7);
	       s[k] = cs_factor[i][k] * group5_table[n][0];
	       s[k + 1] = cs_factor[i][k + 1] * group5_table[n][0];
	       s[k + 64] = cs_factor[i][k] * group5_table[n][1];
	       s[k + 64 + 1] = cs_factor[i][k + 1] * group5_table[n][1];
	       s[k + 128] = cs_factor[i][k] * group5_table[n][2];
	       s[k + 128 + 1] = cs_factor[i][k + 1] * group5_table[n][2];
	       k++;		/* skip right chan dispatch */
	       goto dispatch;
	    case 18 + 3:
	       UNPACKJ_N(3)	/* 7 levels */
	    case 18 + 4:	/* 9 levels grouped 10 bits */
	       n = bitget(10);
	       s[k] = cs_factor[i][k] * group9_table[n][0];
	       s[k + 1] = cs_factor[i][k + 1] * group9_table[n][0];
	       s[k + 64] = cs_factor[i][k] * group9_table[n][1];
	       s[k + 64 + 1] = cs_factor[i][k + 1] * group9_table[n][1];
	       s[k + 128] = cs_factor[i][k] * group9_table[n][2];
	       s[k + 128 + 1] = cs_factor[i][k + 1] * group9_table[n][2];
	       k++;		/* skip right chan dispatch */
	       goto dispatch;
	    case 18 + 5:
	       UNPACKJ_N(4)	/* 15 levels */
	    case 18 + 6:
	       UNPACKJ_N(5)	/* 31 levels */
	    case 18 + 7:
	       UNPACKJ_N(6)	/* 63 levels */
	    case 18 + 8:
	       UNPACKJ_N(7)	/* 127 levels */
	    case 18 + 9:
	       UNPACKJ_N(8)	/* 255 levels */
	    case 18 + 10:
	       UNPACKJ_N(9)	/* 511 levels */
	    case 18 + 11:
	       UNPACKJ_N(10)	/* 1023 levels */
	    case 18 + 12:
	       UNPACKJ_N(11)	/* 2047 levels */
	    case 18 + 13:
	       UNPACKJ_N(12)	/* 4095 levels */
	    case 18 + 14:
	       UNPACKJ_N(13)	/* 8191 levels */
	    case 18 + 15:
	       UNPACKJ_N(14)	/* 16383 levels */
	    case 18 + 16:
	       UNPACKJ_N(15)	/* 32767 levels */
	    case 18 + 17:
	       UNPACKJ_N(16)	/* 65535 levels */
/* -- end of dispatch -- */
	    case 37:
	       bitget_skip(bit_skip);
	    case 36:
	       s += 3 * 64;
	 }			/* end switch */
      }				/* end j loop */
   }				/* end i loop */


}



void L2table_init()
{
	int i, j, code;
	long stepL2[18] = {
		0, 3, 5, 7, 9, 15, 31, 63, 127,
		255, 511, 1023, 2047, 4095, 8191, 16383, 32767, 65535
	};
	//c_values (dequant)
	for (i = 1; i < 18; i++) {
		look_c_valueL2[i] = 2.0F / stepL2[i];
	}
	//scale factor table, scale by 32768 for 16 pcm output
	for (i = 0; i < 64; i++) {
		sf_table[i] = (float) (32768.0 * 2.0 * pow(2.0, -i / 3.0));
	}
	//grouped 3 level lookup table 5 bit token
	for (i = 0; i < 32; i++) {
		code = i;
		for (j = 0; j < 3; j++) {
			group3_table[i][j] = (char) ((code % 3) - 1);
			code /= 3;
		}
	}
	//grouped 5 level lookup table 7 bit token
	for (i = 0; i < 128; i++) {
		code = i;
		for (j = 0; j < 3; j++) {
			group5_table[i][j] = (char) ((code % 5) - 2);
			code /= 5;
		}
	}
	//grouped 9 level lookup table 10 bit token
	for (i = 0; i < 1024; i++) {
		code = i;
		for (j = 0; j < 3; j++) {
			group9_table[i][j] = (short) ((code % 9) - 4);
			code /= 9;
		}
	}
}


int L2decode_start()
{
	MPEG_HEADER *h = &gMpegHeader;
	int i, j, k, bit_code, limit;
	int abcd_index;

/* compute abcd index for bit allo table selection */
   if (h->version)			/* mpeg 1 */
      abcd_index = lookqt[h->mode][h->sr_index][h->br_index];
   else
      abcd_index = 4;		/* mpeg 2 */

   if (abcd_index < 0)
      return 0;			// fail invalid Layer II bit rate index

   for (i = 0; i < 4; i++)
      for (j = 0; j < 16; j++)
	 bat[i][j] = look_bat[abcd_index][i][j];
   for (i = 0; i < 4; i++)
      nbat[i] = look_nbat[abcd_index][i];
   max_sb = nbat[0] + nbat[1] + nbat[2] + nbat[3];
/*----- compute nsb_limit --------*/
//   samprate = sr_table[4 * h->version + h->sr_index];
    if(h->samplingRate != 0){
       nsb_limit = (gDecOption.freqLimit * 64L + h->samplingRate / 2) / h->samplingRate;
    }else{
        nsb_limit = 0;
        return 0;
    }       
/*- caller limit -*/
/*---- limit = 0.94*(32>>reduction_code);  ----*/
   limit = (32 >> gDecOption.reduction);
   if (limit > 8)
      limit--;
   if (nsb_limit > limit)
      nsb_limit = limit;
   if (nsb_limit > max_sb)
      nsb_limit = max_sb;

	if (h->mode != 3)
	{				/* adjust for 2 channel modes */
		for (i = 0; i < 4; i++)
			nbat[i] *= 2;
		max_sb *= 2;
		nsb_limit *= 2;
	}

/* set sbt function */
	bit_code = (gDecOption.convert & 8) ? 1 : 0;
	k = (h->mode == 3) ? 0 : (1 + gDecOption.convert);
	sbt = sbt_table[bit_code][gDecOption.reduction][k];//[2][3][5]
/* clear sample buffer, unused sub bands must be 0 */
	for (i = 0; i < 2304; i++)
		sample[i] = 0.0F;
	return 1;
}

IN_OUT L2decode_frame(unsigned char *bs, unsigned short *pcm)
{
	MPEG_HEADER* h = &gMpegHeader;
	IN_OUT in_out;
//    signed short *pcm = (signed short *)pcm1;
//    unsigned char *pcm = (unsigned char *)pcm1;

/* test sync */
	if ((bs[0] != 0xFF) || (bs[1] & 0xF0) != 0xF0) {
		in_out.inputSize = 0;
		in_out.outputSize = 0;
		return in_out;
	}
	h->crcDisable		= (bs[1] & 0x01);
	h->padding			= (bs[2] & 0x02) >> 1;
	h->mode				= (bs[3] & 0xc0) >> 6;
	h->modeExtension	= (bs[3] & 0x30) >> 4;
//	h->emphasis			= (bs[3] & 0x03);
	if (h->crcDisable) {
		bitget_init(bs + 4);
	}
	else {
		bitget_init(bs + 6);
	}

	stereo_sb = look_joint[(h->mode << 2) + h->modeExtension];
	unpack_ba();			/* unpack bit allocation */
	unpack_sfs();		/* unpack scale factor selectors */
	unpack_sf();			/* unpack scale factor */
	unpack_samp();		/* unpack samples */

	sbt(sample, pcm, 36);

	if (!h->padding) {
		if ((bs[gDecInfo.minFrameSize] != 0xFF) || 
				(bs[gDecInfo.minFrameSize + 1] & 0xF0) != 0xF0) {
			h->padding = 1;
		}
	}
	in_out.inputSize = gDecInfo.minFrameSize + h->padding;
	in_out.outputSize = gDecInfo.maxOutputSize;

	return in_out;
}
