#include <stdlib.h>	//min
#include <string.h>	//memmove
#include <math.h>

#include "bstream.h"
#include "decl3.h"
#include "sbt.h"
#include "mpegdecode.h"

#define min(a,b) ((a < b)?a:b)

extern MPEG_HEADER		gMpegHeader;
extern DECODE_OPTION	gDecOption;
extern DECODE_INFO		gDecInfo;

//orito s
static const int br_table[2][16] =
{	// MPEG-2 Layer3, 2, 1
	{ 0, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 144000, 160000, 0 },
	// MPEG-1 Layer3, 2, 1
	{ 0, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 0 }
};

static const int sr_table[2][4] =
{	// MPEG-2
	{ 22050, 24000, 16000, 0 },
	// MPEG-1
	{ 44100, 48000, 32000, 0 }
};
//orito e

static SBT_FUNCTION sbt_L3 = (SBT_FUNCTION)sbt_dual_L3;

void Xform_dual(void *pcm, int igr);
typedef void (*XFORM_FUNCTION) (void *pcm, int igr);
static XFORM_FUNCTION Xform = Xform_dual;

int nsb_limit = 32;
/*- sample union of int/float  sample[ch][gr][576] */
SAMPLE sample[2][2][576];

int band_limit_nsb = 32;	/* global for hybrid */

/*-- global band tables */
/*-- short portion is 3*x !! --*/
int nBand[2][22];		/* [long/short][cb] */
int sfBandIndex[2][22];		/* [long/short][cb] */


/*====================================================================*/
static int band_limit = (576);
static int band_limit21 = (576);	// limit for sf band 21

static int band_limit12 = (576);	// limit for sf band 12 short

static int gain_adjust = 0;	/* adjust gain e.g. cvt to mono */
static int ncbl_mixed;		/* number of long cb's in mixed block 8 or 6 */
//work
static int nchan;
static int ms_mode;
static int is_mode;
static unsigned int zero_level_pcm = 0;

/* cb_info[igr][ch], compute by dequant, used by joint */
static CB_INFO cb_info[2][2];
static IS_SF_INFO is_sf_info;	/* MPEG-2 intensity stereo */

/*---------------------------------*/
/* main data bit buffer */
#define NBUF (8*1024)
#define BUF_TRIGGER (NBUF-1500)
static unsigned char buf[NBUF];
static int buf_ptr0 = 0;
static int buf_ptr1 = 0;
static int main_pos_bit;

/*---------------------------------*/
static SIDE_INFO side_info;

static SCALEFACT sf[2][2];	/* [gr][ch] */

static int nsamp[2][2];		/* must start = 0, for nsamp[igr_prev] */
static float yout[576];		/* hybrid out, sbt in */

/*====================================================================*/
int hybrid(void *xin, void *xprev, float *y,
	   int btype, int nlong, int ntot, int nprev);
int hybrid_sum(void *xin, void *xin_left, float *y,
	       int btype, int nlong, int ntot);
void sum_f_bands(void *a, void *b, int n);
void FreqInvert(float *y, int n); /* xform, */
void antialias(void *x, int n);
void ms_process(void *x, int n);	/* sum-difference stereo */

void is_process_MPEG1(void *x,	/* intensity stereo */
		      SCALEFACT * sf,
		      CB_INFO cb_info[2],	/* [ch] */
		      int nsamp, int ms_mode);
void is_process_MPEG2(void *x,	/* intensity stereo */
		      SCALEFACT * sf,
		      CB_INFO cb_info[2],	/* [ch] */
		      IS_SF_INFO * is_sf_info,
		      int nsamp, int ms_mode);

void unpack_huff(void *xy, int n, int ntable);
int unpack_huff_quad(void *vwxy, int n, int nbits, int ntable);
void dequant(SAMPLE sample[], int *nsamp,
	     SCALEFACT * sf,
	     GR * gr,
	     CB_INFO * cb_info, int ncbl_mixed);
void unpack_sf_sub_MPEG1(SCALEFACT * scalefac, GR * gr,
			 int scfsi,	/* bit flag */
			 int igr);
void unpack_sf_sub_MPEG2(SCALEFACT sf[],	/* return intensity scale */
			 GR * grdat,
			 int is_and_ch, IS_SF_INFO * is_sf_info);

/*====================================================================*/
void Xform_mono(void *pcm, int igr)
{
   int igr_prev, n1, n2;

/*--- hybrid + sbt ---*/
   n1 = n2 = nsamp[igr][0];	/* total number bands */
   if (side_info.gr[igr][0].block_type == 2)
   {				/* long bands */
      n1 = 0;
      if (side_info.gr[igr][0].mixed_block_flag)
	 n1 = sfBandIndex[0][ncbl_mixed - 1];
   }
   if (n1 > band_limit)
      n1 = band_limit;
   if (n2 > band_limit)
      n2 = band_limit;
   igr_prev = igr ^ 1;

   nsamp[igr][0] = hybrid(sample[0][igr], sample[0][igr_prev],
	 yout, side_info.gr[igr][0].block_type, n1, n2, nsamp[igr_prev][0]);
   FreqInvert(yout, nsamp[igr][0]);
   sbt_L3(yout, pcm, 0);

}
/*--------------------------------------------------------------------*/
void Xform_dual_right(void *pcm, int igr)
{
   int igr_prev, n1, n2;

/*--- hybrid + sbt ---*/
   n1 = n2 = nsamp[igr][1];	/* total number bands */
   if (side_info.gr[igr][1].block_type == 2)
   {				/* long bands */
      n1 = 0;
      if (side_info.gr[igr][1].mixed_block_flag)
	 n1 = sfBandIndex[0][ncbl_mixed - 1];
   }
   if (n1 > band_limit)
      n1 = band_limit;
   if (n2 > band_limit)
      n2 = band_limit;
   igr_prev = igr ^ 1;
   nsamp[igr][1] = hybrid(sample[1][igr], sample[1][igr_prev],
	 yout, side_info.gr[igr][1].block_type, n1, n2, nsamp[igr_prev][1]);
   FreqInvert(yout, nsamp[igr][1]);
   sbt_L3(yout, pcm, 0);

}
/*--------------------------------------------------------------------*/
void Xform_dual(void *pcm, int igr)
{
   int ch;
   int igr_prev, n1, n2;

/*--- hybrid + sbt ---*/
   igr_prev = igr ^ 1;
   for (ch = 0; ch < nchan; ch++)
   {
      n1 = n2 = nsamp[igr][ch];	/* total number bands */
      if (side_info.gr[igr][ch].block_type == 2)
      {				/* long bands */
	 n1 = 0;
	 if (side_info.gr[igr][ch].mixed_block_flag)
	    n1 = sfBandIndex[0][ncbl_mixed - 1];
      }
      if (n1 > band_limit)
	 n1 = band_limit;
      if (n2 > band_limit)
	 n2 = band_limit;
      nsamp[igr][ch] = hybrid(sample[ch][igr], sample[ch][igr_prev],
       yout, side_info.gr[igr][ch].block_type, n1, n2, nsamp[igr_prev][ch]);
      FreqInvert(yout, nsamp[igr][ch]);
      sbt_L3(yout, pcm, ch);
   }

}
/*--------------------------------------------------------------------*/
void Xform_dual_mono(void *pcm, int igr)
{
   int igr_prev, n1, n2, n3;

/*--- hybrid + sbt ---*/
   igr_prev = igr ^ 1;
   if ((side_info.gr[igr][0].block_type == side_info.gr[igr][1].block_type)
       && (side_info.gr[igr][0].mixed_block_flag == 0)
       && (side_info.gr[igr][1].mixed_block_flag == 0))
   {

      n2 = nsamp[igr][0];	/* total number bands max of L R */
      if (n2 < nsamp[igr][1])
	 n2 = nsamp[igr][1];
      if (n2 > band_limit)
	 n2 = band_limit;
      n1 = n2;			/* n1 = number long bands */
      if (side_info.gr[igr][0].block_type == 2)
	 n1 = 0;
      sum_f_bands(sample[0][igr], sample[1][igr], n2);
      n3 = nsamp[igr][0] = hybrid(sample[0][igr], sample[0][igr_prev],
	 yout, side_info.gr[igr][0].block_type, n1, n2, nsamp[igr_prev][0]);
   }
   else
   {				/* transform and then sum (not tested - never happens in test) */
/*-- left chan --*/
      n1 = n2 = nsamp[igr][0];	/* total number bands */
      if (side_info.gr[igr][0].block_type == 2)
      {
	 n1 = 0;		/* long bands */
	 if (side_info.gr[igr][0].mixed_block_flag)
	    n1 = sfBandIndex[0][ncbl_mixed - 1];
      }
      n3 = nsamp[igr][0] = hybrid(sample[0][igr], sample[0][igr_prev],
	 yout, side_info.gr[igr][0].block_type, n1, n2, nsamp[igr_prev][0]);
/*-- right chan --*/
      n1 = n2 = nsamp[igr][1];	/* total number bands */
      if (side_info.gr[igr][1].block_type == 2)
      {
	 n1 = 0;		/* long bands */
	 if (side_info.gr[igr][1].mixed_block_flag)
	    n1 = sfBandIndex[0][ncbl_mixed - 1];
      }
      nsamp[igr][1] = hybrid_sum(sample[1][igr], sample[0][igr],
			     yout, side_info.gr[igr][1].block_type, n1, n2);
      if (n3 < nsamp[igr][1])
	 n1 = nsamp[igr][1];
   }

/*--------*/
   FreqInvert(yout, n3);
   sbt_L3(yout, pcm, 0);
}

static int unpack_side_MPEG1()
{
   int igr, ch;
   int side_bytes;

/* decode partial header plus initial side info */
	side_info.mode = gMpegHeader.mode;
	if (side_info.mode != 1)
		side_info.mode_ext = 0;
	else
	   side_info.mode_ext = gMpegHeader.modeExtension;

/* adjust global gain in ms mode to avoid having to mult by 1/sqrt(2) */
   ms_mode = side_info.mode_ext >> 1;
   is_mode = side_info.mode_ext & 1;

   side_info.main_data_begin = bitget(9);
   if (side_info.mode == 3)
   {
      side_info.private_bits = bitget(5);
      nchan = 1;
      side_bytes = (4 + 17);
/*-- with header --*/
   }
   else
   {
      side_info.private_bits = bitget(3);
      nchan = 2;
      side_bytes = (4 + 32);
/*-- with header --*/
   }
   for (ch = 0; ch < nchan; ch++)
      side_info.scfsi[ch] = bitget(4);
/* this always 0 (both igr) for short blocks */

   for (igr = 0; igr < 2; igr++)
   {
      for (ch = 0; ch < nchan; ch++)
      {
	 side_info.gr[igr][ch].part2_3_length = bitget(12);
	 side_info.gr[igr][ch].big_values = bitget(9);
	 side_info.gr[igr][ch].global_gain = bitget(8) + gain_adjust;
	 if (ms_mode)
	    side_info.gr[igr][ch].global_gain -= 2;
	 side_info.gr[igr][ch].scalefac_compress = bitget(4);
	 side_info.gr[igr][ch].window_switching_flag = bitget(1);
	 if (side_info.gr[igr][ch].window_switching_flag)
	 {
	    side_info.gr[igr][ch].block_type = bitget(2);
	    side_info.gr[igr][ch].mixed_block_flag = bitget(1);
	    side_info.gr[igr][ch].table_select[0] = bitget(5);
	    side_info.gr[igr][ch].table_select[1] = bitget(5);
	    side_info.gr[igr][ch].subblock_gain[0] = bitget(3);
	    side_info.gr[igr][ch].subblock_gain[1] = bitget(3);
	    side_info.gr[igr][ch].subblock_gain[2] = bitget(3);
	  /* region count set in terms of long block cb's/bands */
	  /* r1 set so r0+r1+1 = 21 (lookup produces 576 bands ) */
	  /* if(window_switching_flag) always 36 samples in region0 */
	    side_info.gr[igr][ch].region0_count = (8 - 1);	/* 36 samples */
	    side_info.gr[igr][ch].region1_count = 20 - (8 - 1);
	 }
	 else
	 {
	    side_info.gr[igr][ch].mixed_block_flag = 0;
	    side_info.gr[igr][ch].block_type = 0;
	    side_info.gr[igr][ch].table_select[0] = bitget(5);
	    side_info.gr[igr][ch].table_select[1] = bitget(5);
	    side_info.gr[igr][ch].table_select[2] = bitget(5);
	    side_info.gr[igr][ch].region0_count = bitget(4);
	    side_info.gr[igr][ch].region1_count = bitget(3);
	 }
	 side_info.gr[igr][ch].preflag = bitget(1);
	 side_info.gr[igr][ch].scalefac_scale = bitget(1);
	 side_info.gr[igr][ch].count1table_select = bitget(1);
      }
   }

/* return  bytes in header + side info */
   return side_bytes;
}

static int unpack_side_MPEG2(int igr)
{
   int ch;
   int side_bytes;

/* decode partial header plus initial side info */
	side_info.mode = gMpegHeader.mode;
	if (side_info.mode != 1)
		side_info.mode_ext = 0;
	else
		side_info.mode_ext = gMpegHeader.modeExtension;

/* adjust global gain in ms mode to avoid having to mult by 1/sqrt(2) */
   ms_mode = side_info.mode_ext >> 1;
   is_mode = side_info.mode_ext & 1;

   side_info.main_data_begin = bitget(8);
   if (side_info.mode == 3)
   {
      side_info.private_bits = bitget(1);
      nchan = 1;
      side_bytes = (4 + 9);
/*-- with header --*/
   }
   else
   {
      side_info.private_bits = bitget(2);
      nchan = 2;
      side_bytes = (4 + 17);
/*-- with header --*/
   }
   side_info.scfsi[1] = side_info.scfsi[0] = 0;


   for (ch = 0; ch < nchan; ch++)
   {
      side_info.gr[igr][ch].part2_3_length = bitget(12);
      side_info.gr[igr][ch].big_values = bitget(9);
      side_info.gr[igr][ch].global_gain = bitget(8) + gain_adjust;
      if (ms_mode)
	 side_info.gr[igr][ch].global_gain -= 2;
      side_info.gr[igr][ch].scalefac_compress = bitget(9);
      side_info.gr[igr][ch].window_switching_flag = bitget(1);
      if (side_info.gr[igr][ch].window_switching_flag)
      {
	 side_info.gr[igr][ch].block_type = bitget(2);
	 side_info.gr[igr][ch].mixed_block_flag = bitget(1);
	 side_info.gr[igr][ch].table_select[0] = bitget(5);
	 side_info.gr[igr][ch].table_select[1] = bitget(5);
	 side_info.gr[igr][ch].subblock_gain[0] = bitget(3);
	 side_info.gr[igr][ch].subblock_gain[1] = bitget(3);
	 side_info.gr[igr][ch].subblock_gain[2] = bitget(3);
       /* region count set in terms of long block cb's/bands  */
       /* r1 set so r0+r1+1 = 21 (lookup produces 576 bands ) */
       /* bt=1 or 3       54 samples */
       /* bt=2 mixed=0    36 samples */
       /* bt=2 mixed=1    54 (8 long sf) samples? or maybe 36 */
       /* region0 discussion says 54 but this would mix long */
       /* and short in region0 if scale factors switch */
       /* at band 36 (6 long scale factors) */
	 if ((side_info.gr[igr][ch].block_type == 2))
	 {
	    side_info.gr[igr][ch].region0_count = (6 - 1);	/* 36 samples */
	    side_info.gr[igr][ch].region1_count = 20 - (6 - 1);
	 }
	 else
	 {			/* long block type 1 or 3 */
	    side_info.gr[igr][ch].region0_count = (8 - 1);	/* 54 samples */
	    side_info.gr[igr][ch].region1_count = 20 - (8 - 1);
	 }
      }
      else
      {
	 side_info.gr[igr][ch].mixed_block_flag = 0;
	 side_info.gr[igr][ch].block_type = 0;
	 side_info.gr[igr][ch].table_select[0] = bitget(5);
	 side_info.gr[igr][ch].table_select[1] = bitget(5);
	 side_info.gr[igr][ch].table_select[2] = bitget(5);
	 side_info.gr[igr][ch].region0_count = bitget(4);
	 side_info.gr[igr][ch].region1_count = bitget(3);
      }
      side_info.gr[igr][ch].preflag = 0;
      side_info.gr[igr][ch].scalefac_scale = bitget(1);
      side_info.gr[igr][ch].count1table_select = bitget(1);
   }

/* return  bytes in header + side info */
   return side_bytes;
}

static void unpack_main(unsigned char *pcm, int igr)
{
   int ch;
   int bit0;
   int n1, n2, n3, n4, nn2, nn3;
   int nn4;
   int qbits;
   int m0;

	for (ch = 0; ch < nchan; ch++)
	{
		bitget_init(buf + (main_pos_bit >> 3));
		bit0 = (main_pos_bit & 7);
		if (bit0)
			bitget(bit0);
		main_pos_bit += side_info.gr[igr][ch].part2_3_length;
		bitget_init_end(buf + ((main_pos_bit + 39) >> 3));
/*-- scale factors --*/
		if (gMpegHeader.version)
			unpack_sf_sub_MPEG1(&sf[igr][ch],
				&side_info.gr[igr][ch], side_info.scfsi[ch], igr);
		else
			unpack_sf_sub_MPEG2(&sf[igr][ch],
				&side_info.gr[igr][ch], is_mode & ch, &is_sf_info);
/*--- huff data ---*/
		n1 = sfBandIndex[0][side_info.gr[igr][ch].region0_count];
		n2 = sfBandIndex[0][side_info.gr[igr][ch].region0_count
				+ side_info.gr[igr][ch].region1_count + 1];
		n3 = side_info.gr[igr][ch].big_values;
		n3 = n3 + n3;

		if (n3 > band_limit)
			n3 = band_limit;
		if (n2 > n3)
			n2 = n3;
		if (n1 > n3)
			n1 = n3;
		nn3 = n3 - n2;
		nn2 = n2 - n1;
		unpack_huff(sample[ch][igr], n1, side_info.gr[igr][ch].table_select[0]);
		unpack_huff(sample[ch][igr] + n1, nn2, side_info.gr[igr][ch].table_select[1]);
		unpack_huff(sample[ch][igr] + n2, nn3, side_info.gr[igr][ch].table_select[2]);
		qbits = side_info.gr[igr][ch].part2_3_length - (bitget_bits_used() - bit0);
		nn4 = unpack_huff_quad(sample[ch][igr] + n3, band_limit - n3, qbits,
			     side_info.gr[igr][ch].count1table_select);
		n4 = n3 + nn4;
		nsamp[igr][ch] = n4;
		//limit n4 or allow deqaunt to sf band 22
		if (side_info.gr[igr][ch].block_type == 2)
			n4 = min(n4, band_limit12);
		else
			n4 = min(n4, band_limit21);
		if (n4 < 576)
			memset(sample[ch][igr] + n4, 0, sizeof(SAMPLE) * (576 - n4));
		if (bitget_overrun())
			memset(sample[ch][igr], 0, sizeof(SAMPLE) * (576));
	}
/*--- dequant ---*/
   for (ch = 0; ch < nchan; ch++)
   {
      dequant(sample[ch][igr],
	      &nsamp[igr][ch],	/* nsamp updated for shorts */
	      &sf[igr][ch], &side_info.gr[igr][ch],
	      &cb_info[igr][ch], ncbl_mixed);
   }

/*--- ms stereo processing  ---*/
   if (ms_mode)
   {
      if (is_mode == 0)
      {
	 m0 = nsamp[igr][0];	/* process to longer of left/right */
	 if (m0 < nsamp[igr][1])
	    m0 = nsamp[igr][1];
      }
      else
      {				/* process to last cb in right */
	 m0 = sfBandIndex[cb_info[igr][1].cbtype][cb_info[igr][1].cbmax];
      }
      ms_process(sample[0][igr], m0);
   }
/*--- is stereo processing  ---*/
	if (is_mode)
	{
		if (gMpegHeader.version)
			is_process_MPEG1(sample[0][igr], &sf[igr][1],
					cb_info[igr], nsamp[igr][0], ms_mode);
		else
			is_process_MPEG2(sample[0][igr], &sf[igr][1],
					cb_info[igr], &is_sf_info, nsamp[igr][0], ms_mode);
	}

/*-- adjust ms and is modes to max of left/right */
   if (side_info.mode_ext)
   {
      if (nsamp[igr][0] < nsamp[igr][1])
	 nsamp[igr][0] = nsamp[igr][1];
      else
	 nsamp[igr][1] = nsamp[igr][0];
   }

/*--- antialias ---*/
   for (ch = 0; ch < nchan; ch++)
   {
      if (cb_info[igr][ch].ncbl == 0)
	 continue;		/* have no long blocks */
      if (side_info.gr[igr][ch].mixed_block_flag)
	 n1 = 1;		/* 1 -> 36 samples */
      else
	 n1 = (nsamp[igr][ch] + 7) / 18;
      if (n1 > 31)
	 n1 = 31;
      antialias(sample[ch][igr], n1);
      n1 = 18 * n1 + 8;		/* update number of samples */
      if (n1 > nsamp[igr][ch])
	 nsamp[igr][ch] = n1;
   }
/*--- hybrid + sbt ---*/
   Xform(pcm, igr);

}

static SBT_FUNCTION sbt_tableL3[2][3][2] =
{
    {
        {
   (SBT_FUNCTION) sbt_mono_L3,
   (SBT_FUNCTION) sbt_dual_L3,
        },{
   (SBT_FUNCTION) sbt16_mono_L3,
   (SBT_FUNCTION) sbt16_dual_L3,
        },{
   (SBT_FUNCTION) sbt8_mono_L3,
   (SBT_FUNCTION) sbt8_dual_L3,
        }
/*-- 8 bit output -*/
    },{
        {
   (SBT_FUNCTION) sbtB_mono_L3,
   (SBT_FUNCTION) sbtB_dual_L3,
        },{
   (SBT_FUNCTION) sbtB16_mono_L3,
   (SBT_FUNCTION) sbtB16_dual_L3,
        },{
   (SBT_FUNCTION) sbtB8_mono_L3,
   (SBT_FUNCTION) sbtB8_dual_L3,
        }
    }
};

void Xform_mono(void *pcm, int igr);
void Xform_dual(void *pcm, int igr);
void Xform_dual_mono(void *pcm, int igr);
void Xform_dual_right(void *pcm, int igr);

static XFORM_FUNCTION xform_table[5] =
{
   Xform_mono,
   Xform_dual,
   Xform_dual_mono,
   Xform_mono,			/* left */
   Xform_dual_right,
};

static const struct
{
   int l[23];
   int s[14];
}
sfBandIndexTable[3][3] =
{
/* mpeg-2 */
   {
      {
	 {
	    0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 284, 336, 396, 464, 522, 576
	 }
	 ,
	 {
	    0, 4, 8, 12, 18, 24, 32, 42, 56, 74, 100, 132, 174, 192
	 }
      }
      ,
      {
	 {
	    0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 114, 136, 162, 194, 232, 278, 332, 394, 464, 540, 576
	 }
	 ,
	 {
	    0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 136, 180, 192
	 }
      }
      ,
      {
	 {
	    0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 284, 336, 396, 464, 522, 576
	 }
	 ,
	 {
	    0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192
	 }
      },
   }
   ,
/* mpeg-1 */
   {
      {
	 {
	    0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 52, 62, 74, 90, 110, 134, 162, 196, 238, 288, 342, 418, 576
	 },
	 {
	    0, 4, 8, 12, 16, 22, 30, 40, 52, 66, 84, 106, 136, 192
	 }
      },
      {
	 {
	    0, 4, 8, 12, 16, 20, 24, 30, 36, 42, 50, 60, 72, 88, 106, 128, 156, 190, 230, 276, 330, 384, 576
	 },
	 {
	    0, 4, 8, 12, 16, 22, 28, 38, 50, 64, 80, 100, 126, 192
	 }
      },
      {
	 {
	    0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 54, 66, 82, 102, 126, 156, 194, 240, 296, 364, 448, 550, 576
	 },
	 {
	    0, 4, 8, 12, 16, 22, 30, 42, 58, 78, 104, 138, 180, 192
	 }
      }
   },

/* mpeg-2.5, 11 & 12 KHz seem ok, 8 ok */
   {
      {
	 {
	    0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 284, 336, 396, 464, 522, 576
	 },
	 {
	    0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192
	 }
      },
      {
	 {
	    0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 284, 336, 396, 464, 522, 576
	 },
	 {
	    0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192
	 }
      },
// this 8khz table, and only 8khz, from mpeg123)
      {
	 {
	    0, 12, 24, 36, 48, 60, 72, 88, 108, 132, 160, 192, 232, 280, 336, 400, 476, 566, 568, 570, 572, 574, 576
	 },
	 {
	    0, 8, 16, 24, 36, 52, 72, 96, 124, 160, 162, 164, 166, 192
	 }
      },
   },
};

void quant_init();
void alias_init();
void msis_init_MPEG1();
void msis_init_MPEG2();
void fdct_init();
void imdct_init();
void hwin_init();

void L3table_init()
{
	quant_init();
	alias_init();
	msis_init_MPEG1();
	msis_init_MPEG2();
	fdct_init();
	imdct_init();
	hwin_init();
}

int L3decode_start()
{
	MPEG_HEADER *h = &gMpegHeader;
	int i, j, k;
	int limit;
	int bit_code;

	buf_ptr0 = 0;
	buf_ptr1 = 0;

	if (h->version)
		ncbl_mixed = 8;		/* mpeg-1 */
	else
		ncbl_mixed = 6;		/* mpeg-2 */

/*----- compute nsb_limit --------*/
    if(h->samplingRate != 0){
    	nsb_limit = (gDecOption.freqLimit * 64L + h->samplingRate / 2) / h->samplingRate;
    }else{
        nsb_limit = 0;
    }        
/*- caller limit -*/
	limit = (32 >> gDecOption.reduction);
	if (limit > 8)
		limit--;
	if (nsb_limit > limit)
		nsb_limit = limit;
	limit = 18 * nsb_limit;

	k = h->version;
//	if ((h->sync & 1) == 0)
//		k = 2;			// mpeg 2.5 

	if (k == 1)
	{
		band_limit12 = 3 * sfBandIndexTable[k][h->sr_index].s[13];
		band_limit = band_limit21 = sfBandIndexTable[k][h->sr_index].l[22];
	}
	else
	{
		band_limit12 = 3 * sfBandIndexTable[k][h->sr_index].s[12];
		band_limit = band_limit21 = sfBandIndexTable[k][h->sr_index].l[21];
	}
	band_limit += 8;		/* allow for antialias */
	if (band_limit > limit)
		band_limit = limit;

	if (band_limit21 > band_limit)
		band_limit21 = band_limit;
	if (band_limit12 > band_limit)
		band_limit12 = band_limit;

	band_limit_nsb = (band_limit + 17) / 18;	/* limit nsb's rounded up */
/*----------------------------------------------*/
	gain_adjust = 0;		/* adjust gain e.g. cvt to mono sum channel */
	if ((h->mode != 3) && (gDecOption.convert == 1))
		gain_adjust = -4;

	bit_code = (gDecOption.convert & 8) ? 1 : 0;
//	convert_code = convert_code & 3;	/* higher bits used by dec8 freq cvt */
	k = (h->mode != 3) ? (1 + gDecOption.convert) : 0;
//   sbt_L3 = sbt_table[bit_code][reduction_code][out_chans - 1];
	sbt_L3 = sbt_tableL3[bit_code][gDecOption.reduction][gDecInfo.channels - 1];//[2][3][2]
	Xform = xform_table[k];//[5]

	if (bit_code)
		zero_level_pcm = 128;	/* 8 bit output */
	else
		zero_level_pcm = 0;

/*- init band tables --*/
	k = h->version;
//   if ((h->sync & 1) == 0)
//      k = 2;			// mpeg 2.5 

	for (i = 0; i < 22; i++)
		sfBandIndex[0][i] = sfBandIndexTable[k][h->sr_index].l[i + 1];
	for (i = 0; i < 13; i++)
		sfBandIndex[1][i] = 3 * sfBandIndexTable[k][h->sr_index].s[i + 1];
	for (i = 0; i < 22; i++)
		nBand[0][i] = sfBandIndexTable[k][h->sr_index].l[i + 1]
				- sfBandIndexTable[k][h->sr_index].l[i];
	for (i = 0; i < 13; i++)
		nBand[1][i] = sfBandIndexTable[k][h->sr_index].s[i + 1]
				- sfBandIndexTable[k][h->sr_index].s[i];

/*--- clear buffers --*/
	for (i = 0; i < 576; i++)
		yout[i] = 0.0f;
	for (j = 0; j < 2; j++)
	{
		for (k = 0; k < 2; k++)
		{
			for (i = 0; i < 576; i++)
			{
				sample[j][k][i].x = 0.0f;
				sample[j][k][i].s = 0;
			}
		}
	}
	return 1;
}

void L3decode_reset()
{
	buf_ptr0 = buf_ptr1 = 0;
}

IN_OUT L3decode_frame_MPEG1(unsigned char *bs, unsigned char *pcm)
{
	MPEG_HEADER* h = &gMpegHeader;
	IN_OUT in_out;
	int side_bytes, padframebytes, crcbytes;
	int nbytes;
//    unsigned char *pcm = (unsigned char *)pcm1;

/* test sync */
	if ((bs[0] != 0xFF) || (bs[1] & 0xF0) != 0xF0) {
		in_out.inputSize = 0;
		in_out.outputSize = 0;
		return in_out;
	}
	h->crcDisable		= (bs[1] & 0x01);
	h->padding			= (bs[2] & 0x02) >> 1;

    ////////////////////////////////////////////////////////////
    //orito  s

	h->mode				= (bs[3] & 0xc0) >> 6;
	h->modeExtension	= (bs[3] & 0x30) >> 4;
//	h->emphasis			= (bs[3] & 0x03);

	h->version				= (bs[1] & 0x08) >> 3;
  	h->br_index				= (bs[2] & 0xf0) >> 4;
	h->sr_index				= (bs[2] & 0x0c) >> 2;
	h->bitRate = br_table[h->version][h->br_index];
	h->samplingRate = sr_table[h->version][h->sr_index];
    if(h->bitRate == 0 || h->samplingRate == 0){
		in_out.inputSize = 0;
		in_out.outputSize = 0;
		return in_out;
    }
    //orito  e
    ////////////////////////////////////////////////////////////

	if (h->crcDisable) {
		bitget_init(bs + 4);
		crcbytes = 0;
	}
	else {
		bitget_init(bs + 6);
		crcbytes = 2;
	}

/*-- unpack side info --*/
	side_bytes = unpack_side_MPEG1();

    ////////////////////////////////////////////////////////////
    //orito s
//	padframebytes = gDecInfo.minFrameSize + h->padding;
    padframebytes = (144 * h->bitRate) / h->samplingRate + h->padding;
    //orito e
    ////////////////////////////////////////////////////////////

	in_out.inputSize = padframebytes;


/*-- load main data and update buf pointer --*/
/*------------------------------------------- 
   if start point < 0, must just cycle decoder 
   if jumping into middle of stream, 
w---------------------------------------------*/
	buf_ptr0 = buf_ptr1 - side_info.main_data_begin;	/* decode start point */
	if (buf_ptr1 > BUF_TRIGGER)
	{				/* shift buffer */
		memmove(buf, buf + buf_ptr0, side_info.main_data_begin);
		buf_ptr0 = 0;
		buf_ptr1 = side_info.main_data_begin;
	}
	nbytes = padframebytes - side_bytes - crcbytes;
	memmove(buf + buf_ptr1, bs + side_bytes + crcbytes, nbytes);
	buf_ptr1 += nbytes;
/*-----------------------*/

   if (buf_ptr0 >= 0){
      main_pos_bit = buf_ptr0 << 3;
      unpack_main(pcm, 0);
      unpack_main(pcm + (gDecInfo.maxOutputSize / 2), 1);
      in_out.outputSize = gDecInfo.maxOutputSize;
   }else{
      memset(pcm, zero_level_pcm, gDecInfo.maxOutputSize);	/* fill out skipped frames */
      in_out.outputSize = gDecInfo.maxOutputSize;
   }

   return in_out;
}

IN_OUT L3decode_frame_MPEG2(unsigned char *bs, unsigned char *pcm)
{
	MPEG_HEADER* h = &gMpegHeader;
	IN_OUT in_out;
	int side_bytes, padframebytes, crcbytes;
	int nbytes;
	static int igr = 0;
    static int padtemp = 0,vbr_count=0;  // orito
    int i;                               // orito
//    unsigned char *pcm = (unsigned char *)pcm1;

/* test sync */
	if ((bs[0] != 0xFF) || (bs[1] & 0xF0) != 0xF0) {
		in_out.inputSize = 0;
		in_out.outputSize = 0;
		return in_out;
	}
    
	h->crcDisable		= (bs[1] & 0x01);
	h->padding			= (bs[2] & 0x02) >> 1;

    ////////////////////////////////////////////////////////////
    //orito  s
	h->mode				= (bs[3] & 0xc0) >> 6;
	h->modeExtension	= (bs[3] & 0x30) >> 4;
//	h->emphasis			= (bs[3] & 0x03);

	h->version				= (bs[1] & 0x08) >> 3;
  	h->br_index				= (bs[2] & 0xf0) >> 4;
	h->sr_index				= (bs[2] & 0x0c) >> 2;
	h->bitRate = br_table[h->version][h->br_index];
	h->samplingRate = sr_table[h->version][h->sr_index];
    if(h->bitRate == 0 || h->samplingRate == 0){
		in_out.inputSize = 0;
		in_out.outputSize = 0;
		return in_out;
    }
    //orito  e
    ////////////////////////////////////////////////////////////

	if (h->crcDisable) {
		bitget_init(bs + 4);
		crcbytes = 0;
	}else{
		bitget_init(bs + 6);
		crcbytes = 2;
	}

/*-- unpack side info --*/
	side_bytes = unpack_side_MPEG2(igr);
    ////////////////////////////////////////////////////////////
    //orito s
//    padframebytes = gDecInfo.minFrameSize + h->padding;
    padframebytes = (144 * h->bitRate) / h->samplingRate;
    if(h->version == 0){ //version 2
        padframebytes /= 2;
    }
    padframebytes += h->padding;

    //orito e
    ////////////////////////////////////////////////////////////
	in_out.inputSize = padframebytes;

	buf_ptr0 = buf_ptr1 - side_info.main_data_begin;	/* decode start point */
	if (buf_ptr1 > BUF_TRIGGER)
	{				/* shift buffer */
		memmove(buf, buf + buf_ptr0, side_info.main_data_begin);
		buf_ptr0 = 0;
		buf_ptr1 = side_info.main_data_begin;
	}
	nbytes = padframebytes - side_bytes - crcbytes;
	memmove(buf + buf_ptr1, bs + side_bytes + crcbytes, nbytes);
	buf_ptr1 += nbytes;
/*-----------------------*/

	if (buf_ptr0 >= 0)
	{
		main_pos_bit = buf_ptr0 << 3;
		unpack_main(pcm, igr);
	}else{
		memset(pcm, zero_level_pcm, gDecInfo.maxOutputSize);	/* fill out skipped frames */
	}

    in_out.outputSize = gDecInfo.maxOutputSize;
	igr = igr ^ 1;
	return in_out;
}
