/*____________________________________________________________________________
	
        MP3 Decoder originally Copyright (C) 1995-1997 Xing Technology
        Corp.  http://www.xingtech.com


        Customized by Takayuki Orito (C) 2000 Mitani,Co.,Ltd.
        ܥץϡBorland C++Builder 4(C)1983-1999Inprise Corporation
        ѥ뤹뤿ˡؿȤΰη̾Ūʥ㥹ȡ
        ι¤Τˤ뤿Υå䤹ѹ
        ޤ
____________________________________________________________________________*/

#include <math.h>

#include "mpegdecode.h"


//---------------------------------------------------------------------------
typedef IN_OUT (*DECODE_FUNCTION) (unsigned char *bs, void *pcm);

MPEG_HEADER		gMpegHeader;
DECODE_OPTION	gDecOption;
DECODE_INFO		gDecInfo;

static DECODE_FUNCTION gDecFunc;

static const int br_table[2][3][16] =
{{	// MPEG-2 Layer3, 2, 1
	{ 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },
	{ 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },
	{ 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 }
},{	// MPEG-1 Layer3, 2, 1
	{ 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 },
	{ 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0 },
	{ 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0 }
}};

static const int sr_table[2][4] =
{	// MPEG-2
	{ 22050, 24000, 16000, 0 },	
	// MPEG-1
	{ 44100, 48000, 32000, 0 }
};


void L1table_init();
void L2table_init();
void L3table_init();
void sbt_init();

int L1decode_start();
int L2decode_start();
int L3decode_start();
IN_OUT L1decode_frame(unsigned char *bs, void *pcm);
IN_OUT L2decode_frame(unsigned char *bs, void *pcm);
IN_OUT L3decode_frame_MPEG1(unsigned char *bs, void *pcm);
IN_OUT L3decode_frame_MPEG2(unsigned char *bs, void *pcm);

void L3decode_reset();


void mpegDecodeInit()
{
	L1table_init();
	L2table_init();
	L3table_init();

	gDecOption.reduction = 0;
	gDecOption.convert = 0;
	gDecOption.freqLimit = 24000;
}

int mpegSetDecodeOption(DECODE_OPTION *option)
{
	if (option->freqLimit < 1000) {
		return 0;
	}
	gDecOption = *option;
	return 1;
}

void mpegGetDecodeOption(DECODE_OPTION *option)
{
	*option = gDecOption;
}

int mpegGetHeader(unsigned char *buf, int size, MPEG_HEADER *h, int *sync)
{
	unsigned char *p;
	int i;

	*sync = 0;
	size -= 3;
	if (size <= 0) {
		return 0;
	}
	for (i = 0; i < size; i ++) {
		if ((buf[i] == 0xFF) && ((buf[i + 1] >> 4) == 0x0F)) {
			break;
		}
	}
	if (i == size) {
		return 0;
	}
	p = &buf[i];
	h->version				= (p[1] & 0x08) >> 3;
	h->layer				= (p[1] & 0x06) >> 1;
	h->crcDisable			= (p[1] & 0x01);
	h->br_index				= (p[2] & 0xf0) >> 4;
	h->sr_index				= (p[2] & 0x0c) >> 2;
	h->padding				= (p[2] & 0x02) >> 1;
	h->extension			= (p[2] & 0x01);
	h->mode					= (p[3] & 0xc0) >> 6;
	h->modeExtension		= (p[3] & 0x30) >> 4;
	h->copyright			= (p[3] & 0x08) >> 3;
	h->original				= (p[3] & 0x04) >> 2;
	h->emphasis				= (p[3] & 0x03);
	/* mpeg2.5 */
//		h->version = 2;
	h->bitRate = br_table[h->version][h->layer - 1][h->br_index];
	h->samplingRate = sr_table[h->version][h->sr_index];
	*sync = i;

    if(h->layer == 0 || h->layer > 3)   return 0;
    if(h->bitRate == 0) return 0;
    if(h->samplingRate == 0)    return 0;
    
	return 1;
}

int mpegGetDecodeInfo(MPEG_HEADER *h, DECODE_INFO *info)
{
	int bitRate, samplingRate;
	int frameSize, padSize;
	int outSize;

	bitRate = h->bitRate;
	samplingRate = h->samplingRate;
	switch (h->layer) {
	case 1:	/* layer III */
        if(samplingRate != 0){
    		frameSize = 144 * bitRate * 1000 / samplingRate;
        }else{
            frameSize = 0;
        }
		padSize = 1;
		outSize = 1152 >> gDecOption.reduction;
		switch (h->version) {
		case 0:// mpeg2
            outSize /= 2;
            frameSize /= 2;
			break;
		case 2:// mpeg2.5
            outSize /= 2;
            samplingRate /= 2;
			break;
		}
		break;
	case 2: /* layer II */
        if(samplingRate != 0){
    		frameSize = 144 * bitRate * 1000 / samplingRate;
        }else{
            frameSize = 0;
        }
		padSize = 1;
		outSize = 1152 >> gDecOption.reduction;
		break;
	case 3: /* layer I */
        if(samplingRate != 0){
    		frameSize = (12 * bitRate * 1000 / samplingRate) * 4;
        }else{
            frameSize = 0;
        }
		padSize = 4;
		outSize = 384 >> gDecOption.reduction;
		break;
	default:
		return 0;
	}

	if ((h->mode == 3) || (gDecOption.convert & 3)) {
		info->channels = 1;
	}else{
		info->channels = 2;
	}
	info->samplingRate = samplingRate >> gDecOption.reduction;
	info->minFrameSize = frameSize;
	info->maxFrameSize = frameSize + padSize;
	if (gDecOption.convert & 8) {
		info->bitsPerSample = 8;
		info->maxOutputSize = outSize * info->channels;
	}
	else {
		info->bitsPerSample = 16;
		info->maxOutputSize = outSize * info->channels * 2;
	}
	return 1;
}


int mpegDecodeStart(MPEG_HEADER *h)
{
	if (!mpegGetDecodeInfo(h, &gDecInfo)) {
		return 0;
	}
	gMpegHeader = *h;
	sbt_init();

	switch (h->layer) {
	case 1:	/* layer III */
		if (!L3decode_start()) {
			return 0;
		}
		switch (h->version) {
		case 0:// mpeg2
			gDecFunc = L3decode_frame_MPEG2;
			break;
		case 1:// mpeg1
			gDecFunc = L3decode_frame_MPEG1;
			break;
		case 2:// mpeg2.5
			gDecFunc = L3decode_frame_MPEG2;
			break;
		}
		break;
	case 2: /* layer II */
		if (!L2decode_start()) {
			return 0;
		}
		gDecFunc = L2decode_frame;
		break;
	case 3: /* layer I */
		if (!L1decode_start()) {
			return 0;
		}
		gDecFunc = L1decode_frame;
		break;
	default:
		return 0;
	}
	return 1;
}

IN_OUT mpegDecodeFrame(unsigned char *bs, void *pcm)
{
	return gDecFunc(bs, pcm);
}




void mpegDecodeReset()
{//only layer3
	L3decode_reset();
}

int mpegSkipFrame(unsigned char *bs)
{
	int		padding;
    ////////////////////////////////////////////////////////////
    //orito  s
    MPEG_HEADER h;
    int frameSize;
    //orito  e
    ////////////////////////////////////////////////////////////

	if ((bs[0] != 0xFF) || ((bs[1] >> 4) != 0x0F)) {
		return 0;
	}
	padding	= (bs[2] & 0x02) >> 1;
    
    ////////////////////////////////////////////////////////////
    //orito  s
	h.version				= (bs[1] & 0x08) >> 3;
  	h.br_index				= (bs[2] & 0xf0) >> 4;
	h.sr_index				= (bs[2] & 0x0c) >> 2;
	h.layer			    	= (bs[1] & 0x06) >> 1;
	h.bitRate = br_table[h.version][h.layer - 1][h.br_index];
	h.samplingRate = sr_table[h.version][h.sr_index];
    if(h.bitRate == 0 || h.samplingRate == 0){
		return 0;
    }


//	return gDecInfo.minFrameSize + padding;
	switch (h.layer) {
	case 1:	/* layer III */
        frameSize = 144 * h.bitRate * 1000 /h.samplingRate;
		switch (h.version) {
		case 0:// mpeg2
            frameSize = frameSize / 2;
			break;
		case 2:// mpeg2.5
			break;
		}
        frameSize += 1 * padding;
        //           1:paddingΥ
        break;
	case 2: /* layer II */
        frameSize = (144 * h.bitRate * 1000) / h.samplingRate;
        frameSize += 1 * padding;
        break;
	case 3: /* layer I */
        frameSize = ((12 * h.bitRate * 1000) / h.samplingRate + 4 * padding) * 4;
        //                                                      4:paddingΥ
        break;
    }

    return frameSize;
    //orito  e
    ////////////////////////////////////////////////////////////
}
