/*=========================================================================
    InputMP3 Class
    for BieAmp with ID3Tag Editor
        Copyright (c) 2000-2004 Iori Ayane
      
    MP3 Decoder originally Copyright (C) 1995-1997 Xing Technology
    Corp.  http://www.xingtech.com
=========================================================================*/

//#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../win/win.h"
#include "../win/other.h"
#include "inputmp3.h"


#pragma pack(1)
struct RiffChunk {
  BYTE	chunkID[4];		//'RIFF'
  DWORD	chunkSize;
  BYTE	formType[4];	//'WAVE','RMP3'
};

struct CommonChunk {
  BYTE	chunkID[4];
  DWORD	chunkSize;	
};

struct RMPInfo {
  BYTE	infoID[4];
  DWORD	length;
  BYTE	data[1];
};

struct ID3Tagv1 {
  BYTE	tag[3];
  BYTE	trackName[30];
  BYTE	artistName[30];
  BYTE	albumName[30];
  BYTE	year[4];
  BYTE	comment[30];
  BYTE	genre;
};//128Х

#pragma pack()

char *info_name[] = {"INAM","IART","IPRD","ICMT","ICRD","IGNR","ICOP","IENG",
		     "ISRC","ISFT","IKEY","ITCH","ICMS","ILYC","IMED",
		     "IMP3","ISBJ"};

static void copyRMPInfo(InputInfo* pInfo, RMPInfo* pRMPInfo);
void copyRIFFInfo(RIFFInfo *pInfo,RMPInfo *pRMPInfo);
static void strcpyRMP(char* pszDest, const char* pszSrc, int length);
void strcpyRIFF(char* pszDest, const char* pszSrc, int length);
void freeRIFFInfo(RIFFInfo *pInfo);


BOOL searchListChunk(FILE *pszFile);


#ifndef TAGEDIT_ONLY   //--------  start TAGEDIT_ONLY ----
/////////////////////////////////////////////////////
//  RIFFInfo
BOOL InputMP3::readRIFFInfo(InputInfo* pInfo)
{
  RiffChunk	riff;
  CommonChunk	common;
  BOOL		result;
  DWORD		bytesRead;

  //եƬꡢRIFF󥯤ɤ߹
  ::SetFilePointer(m_hFile, 0, NULL, FILE_BEGIN);
  result = ::ReadFile(m_hFile, &riff, sizeof(riff), &bytesRead, NULL);
  if (!result || bytesRead != sizeof(riff)) {
    return false;
  }
  if (::memcmp(riff.chunkID, "RIFF", 4) != 0) {
    return false;
  }
  m_firstSync = sizeof(riff);
  //data󥯤õ
  for (;;) {
    result = ::ReadFile(m_hFile, &common, sizeof(common), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(common)) {
      return false;
    }
    if (::memcmp(common.chunkID, "data", 4) == 0) {
      //dataȯ
      break;
    }
    else {
      //Υ󥯤ޤɤФ
      m_firstSync = ::SetFilePointer(m_hFile, common.chunkSize, 
				     NULL, FILE_CURRENT);
    }
  }
  m_dataSize = common.chunkSize;
  m_firstSync += bytesRead;
  //RIFF MP3 LIST󥯤õ
  if((common.chunkSize % 2) == 1){
    common.chunkSize++;
  }
  ::SetFilePointer(m_hFile, common.chunkSize, NULL, FILE_CURRENT);
  for (;;) {
    result = ::ReadFile(m_hFile, &common, sizeof(common), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(common)) {
      return true;
    }
    if (::memcmp(common.chunkID, "LIST", 4) == 0) {
      //LISTȯ
      break;
    }
    else {
      //Υ󥯤ޤɤФ
      ::SetFilePointer(m_hFile, common.chunkSize, NULL, FILE_CURRENT);
    }
  }

  loadRMPInfo(m_hFile,pInfo);

  return true;
}
#endif  //---------  end TAGEDIT_ONLY --------------------


////////////////////////////////////////////////////////////////////////
//С饤
BOOL InputMP3::readRIFFInfo(FILE *pszFile,InputInfo* pInfo)
{
  
  if(!searchListChunk(pszFile)){
    return false;
  }
  loadRMPInfo(pszFile,pInfo);
  
  return true;
}



#define CHECK(x)	(::memcmp(pRMPInfo->infoID, x, 4) == 0)
#define SET(x)		strcpyRMP(x, (char*)pRMPInfo->data, pRMPInfo->length)

///////////////////////////////////////////////////////////////
// RIFFInfoɤ߼
void InputMP3::loadRMPInfo(FILE *pszFile,InputInfo* pInfo)
{
  CommonChunk	common;
  BOOL		result;
  DWORD		bytesRead;
  
  BYTE* pRMPList = (BYTE*)::malloc(common.chunkSize);
  if (pRMPList == NULL) {
    return;
  }
  BYTE listID[4];
  result = ::ReadFile(pszFile, listID, 4, &bytesRead, NULL);
  if (!result || bytesRead != 4) {
    return;
  }
  if (::memcmp(listID, "INFO", 4) != 0) {
    //INFOʳ̵
    return;
  }
  
  RMPInfo* pRMPInfo = (RMPInfo*)::malloc(sizeof(RMPInfo));
  if (pRMPInfo == NULL) {
    return;
  }
  DWORD remains = common.chunkSize - 4;
  DWORD baseSize = sizeof(RMPInfo) - sizeof(pRMPInfo->data);
  while (remains >= sizeof(RMPInfo)) {
    result = ::ReadFile(pszFile, pRMPInfo, baseSize, &bytesRead, NULL);
    if (!result || bytesRead != baseSize) {
      break;
    }
    if((pRMPInfo->length % 2) == 1){
      pRMPInfo->length++;
    }
    pRMPInfo = (RMPInfo*)::realloc(pRMPInfo, baseSize + pRMPInfo->length);
    if (pRMPInfo == NULL) {
      break;
    }
    result = ::ReadFile(pszFile, pRMPInfo->data, pRMPInfo->length, &bytesRead, NULL);
    if (!result || bytesRead != pRMPInfo->length) {
      break;
    }
    copyRMPInfo(pInfo, pRMPInfo);
    remains -= baseSize + pRMPInfo->length;
  }
  ::free(pRMPInfo);
}
///////////////////////////////////////////////////////////////
// RIFFInfoɤ߼(С饤)
void InputMP3::loadRMPInfo(FILE *pszFile,RIFFInfo* pInfo)
{
  CommonChunk	common;
  BOOL		result;
  DWORD		bytesRead;


  BYTE* pRMPList = (BYTE*)::malloc(common.chunkSize);
  if (pRMPList == NULL) {
    return;
  }
  BYTE listID[4];
  result = ::ReadFile(pszFile, listID, 4, &bytesRead, NULL);
  if (!result || bytesRead != 4) {
    return;
  }
  if (::memcmp(listID, "INFO", 4) != 0) {
    //INFOʳ̵
    return;
  }
  
  RMPInfo* pRMPInfo = (RMPInfo*)::malloc(sizeof(RMPInfo));
  if (pRMPInfo == NULL) {
    return;
  }
  DWORD remains = common.chunkSize - 4;
  DWORD baseSize = sizeof(RMPInfo) - sizeof(pRMPInfo->data);
  while (remains >= sizeof(RMPInfo)) {
    result = ::ReadFile(pszFile, pRMPInfo, baseSize, &bytesRead, NULL);
    if (!result || bytesRead != baseSize) {
      break;
    }
    if((pRMPInfo->length % 2) == 1){
      pRMPInfo->length++;
    }
    pRMPInfo = (RMPInfo*)::realloc(pRMPInfo, baseSize + pRMPInfo->length);
    if (pRMPInfo == NULL) {
      break;
    }
    result = ::ReadFile(pszFile, pRMPInfo->data, pRMPInfo->length, &bytesRead, NULL);
    if (!result || bytesRead != pRMPInfo->length) {
      break;
    }
    copyRIFFInfo(pInfo, pRMPInfo);
    remains -= baseSize + pRMPInfo->length;
  }
  ::free(pRMPInfo);
}

//////////////////////////////////////////////////////////////
// RIFF Info򥳥ԡ
void copyRMPInfo(InputInfo* pInfo,RMPInfo* pRMPInfo)
{
  if (CHECK("INAM")) {		//track name
    SET(pInfo->szTrackName);
  }else if(CHECK("IART")) {	//artist name
    SET(pInfo->szArtistName);
  }else if(CHECK("ICMT")){
    SET(pInfo->szComment);
  }else if(CHECK("ICRD")){
    SET(pInfo->szYear);
  }else if(CHECK("IPRD")){
    SET(pInfo->szAlbumName);
  }
}
#define SET3(x) strcpyRIFF(x, (char*)pRMPInfo->data, pRMPInfo->length)
#define SET2(x) pInfo->data[x]=(char *)malloc(pRMPInfo->length+1);SET3(pInfo->data[x])
void copyRIFFInfo(RIFFInfo *pInfo,RMPInfo *pRMPInfo)
{
  if (CHECK("INAM")) {		//track name
    pInfo->data[INAM] = (char *)malloc(pRMPInfo->length);
    SET(pInfo->data[INAM]);
  }else if(CHECK("IART")) {	//artist name
    pInfo->data[IART] = (char *)malloc(pRMPInfo->length);
    SET(pInfo->data[IART]);
  }else if(CHECK("ICMT")){
    pInfo->data[ICMT] = (char *)malloc(pRMPInfo->length);
    SET(pInfo->data[ICMT]);
  }else if(CHECK("ICRD")){
    pInfo->data[ICRD] = (char *)malloc(pRMPInfo->length);
    SET(pInfo->data[ICRD]);
  }else if(CHECK("IPRD")){
    pInfo->data[IPRD] = (char *)malloc(pRMPInfo->length);
    SET(pInfo->data[IPRD]);
  }else if(CHECK("IGNR")){
    pInfo->data[IGNR] = (char *)malloc(pRMPInfo->length);
    SET(pInfo->data[IGNR]);
  }else if(CHECK("ICOP")){
    SET2(ICOP);
  }else if(CHECK("IENG")){
    SET2(IENG);
  }else if(CHECK("ISRC")){
    SET2(ISRC);
  }else if(CHECK("ISFT")){
    SET2(ISFT);
  }else if(CHECK("IKEY")){
    SET2(IKEY);
  }else if(CHECK("ITCH")){
    SET2(ITCH);
  }else if(CHECK("ICMS")){
    SET2(ICMS);
  }else if(CHECK("ILYC")){
    SET2(ILYC);
  }else if(CHECK("IMED")){
    SET2(IMED);
  }else if(CHECK("IMP3")){
    SET2(IMP3);
  }else if(CHECK("ISBJ")){
    SET2(ISBJ);
  }
}
void strcpyRMP(char* pszDest, const char* pszSrc, int length)
{
  if (length >= INFO_NAME_SIZE) {
    ::memcpy(pszDest, pszSrc, INFO_NAME_SIZE);//length);
    pszDest[INFO_NAME_SIZE - 1] = 0;
  }else {
    ::memcpy(pszDest, pszSrc, length);
    pszDest[length-1] = 0;
  }
}
void strcpyRIFF(char* pszDest, const char* pszSrc, int length)
{
  ::memcpy(pszDest, pszSrc, length);
  pszDest[length] = 0;
  
}
void freeRIFFInfo(RIFFInfo *pInfo)
{
  for(int i=0;i<17;i++){
    if(pInfo->data[i] != NULL){
      free(pInfo->data[i]);
    }
  }
}
////////////////////////////////////////////////////////////////////////
//  ꥹȥ󥯤õ
BOOL searchListChunk(FILE *pszFile)
{
  RiffChunk	riff;
  CommonChunk	common;
  BOOL		result;
  DWORD		bytesRead;
  int		    firstSync;
  

  //եƬꡢRIFF󥯤ɤ߹
  ::SetFilePointer(pszFile, 0, NULL, FILE_BEGIN);
  result = ::ReadFile(pszFile, &riff, sizeof(riff), &bytesRead, NULL);
  if (!result || bytesRead != sizeof(riff)) {
    return false;
  }
  if (::memcmp(riff.chunkID, "RIFF", 4) != 0) {
    return false;
  }
  firstSync = sizeof(riff);
  //data󥯤õ
  for (;;) {
    result = ::ReadFile(pszFile, &common, sizeof(common), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(common)) {
      return false;
    }
    if (::memcmp(common.chunkID, "data", 4) == 0) {
      //dataȯ
      break;
    }
    else {
      //Υ󥯤ޤɤФ
      firstSync = ::SetFilePointer(pszFile, common.chunkSize, NULL, FILE_CURRENT);
    }
  }
  //	dataSize = common.chunkSize;
  //	firstSync += bytesRead;
  //RIFF MP3 LIST󥯤õ
  if((common.chunkSize % 2) == 1){
    common.chunkSize++;
  }
  ::SetFilePointer(pszFile, common.chunkSize, NULL, FILE_CURRENT);
  for (;;) {
    result = ::ReadFile(pszFile, &common, sizeof(common), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(common)) {
      return false;
    }
    if (::memcmp(common.chunkID, "LIST", 4) == 0) {
      //LISTȯ
      break;
    } else {
      //Υ󥯤ޤɤФ
      ::SetFilePointer(pszFile, common.chunkSize, NULL, FILE_CURRENT);
    }
  }

  return true;

}



//////////////////////////////////////////////////////////////////////////
//ν
DWORD TagLen(char *string);
BOOL WriteTag(FILE *pszFile,char *ID,char *string);

BOOL InputMP3::writeRIFFInfo(char *path,InputInfo *pInfo)
{
  FILE *pszFile = NULL;
  ID3Tagv1	tag;
  DWORD		bytesRead;
  CommonChunk	common;


  pszFile = ::CreateFile(path, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ,
			 NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if(pszFile == NULL) {
    fprintf(stderr,"ե񤭹Ѥ˳ޤǤ\n");
    return false;
  }

  //ꥹȥ󥯤õ
  if(!searchListChunk(pszFile)){
    return false;
  }
  //٤ƤRIFF Info
  RIFFInfo info;
  for(int i=0;i<17;i++){
    info.data[i] = NULL;
  }
  loadRMPInfo(pszFile,&info);

  //ꥹȥ󥯤õ
  if(!searchListChunk(pszFile)){
    return false;
  }

  //եݥ󥿤ꥹȥ󥯤Ƭ᤹
  ::SetFilePointer(pszFile,-1*sizeof(common),NULL,FILE_CURRENT);

  //
  DWORD chunksize = 4;
  //    int num_null=0;

  //󥯥η¬
  chunksize += 8+TagLen(pInfo->szTrackName);
  chunksize += 8+TagLen(pInfo->szArtistName);
  chunksize += 8+TagLen(pInfo->szAlbumName);
  chunksize += 8+TagLen(pInfo->szComment);
  for(int i=4;i<17;i++){
    if(info.data[i] != NULL){
      chunksize += 8+ TagLen(info.data[i]);
    }else{
      //            num_null++;
      //            chunksize += 10;
    }
  }
  //ֺǸ˽񤭤ΤILYC(LYRICS)ΤФ󥯥򣲰
  if(memcmp(info.data[ILYC],"LYRICS",6) == 0){
    chunksize -= 2;
  }else{
    chunksize += 136;
  }

  //񤭤ޤʤΤ˲äʬ
  //    chunksize -= num_null * 10;
    


  //ꥹȥ
  memcpy(common.chunkID,"LIST",4);
  common.chunkSize = chunksize;
  ::WriteFile(pszFile,&common,sizeof(common),&bytesRead,NULL);
  ::WriteFile(pszFile,(void *)("INFO"),4,&bytesRead,NULL);
  
  //ȥ
  WriteTag(pszFile,"INAM",pInfo->szTrackName);
  
  //ƥ
  WriteTag(pszFile,"IART",pInfo->szArtistName);

  //̾ʥХ̾
  WriteTag(pszFile,"IPRD",pInfo->szAlbumName);

  //
  WriteTag(pszFile,"ICMT",pInfo->szComment);

  //ʳΤ
  for(int i=4;i<17;i++){
    if(info.data[i] != NULL){
      if(!WriteTag(pszFile,info_name[i],info.data[i])){
	//falseǵäƤRIFF InfoLyricsߤƤΤ
	SetEndOfFile(pszFile);
	freeRIFFInfo(&info);
	//::CloseHandle(pszFile);
	fclose(pszFile);
	return true;
      }
    }
  }
    
  //ID3TagΥߡ
  memcpy(common.chunkID,"IID3",4);
  common.chunkSize = 128;
  ::WriteFile(pszFile,&common,sizeof(common),&bytesRead,NULL);
  ZeroMemory(&tag,128);
  strcopy((char *)tag.tag,(char *)("TAG"),128);
  ::WriteFile(pszFile,&tag,sizeof(tag),&bytesRead,NULL);


  SetEndOfFile(pszFile);
  
  freeRIFFInfo(&info);
    
  //::CloseHandle(pszFile);
  fclose(pszFile);

  return true;
}

//Ĺޤ
DWORD TagLen(char *string)
{
  DWORD temp;
  if(((temp = strlen(string)+1)%2) == 1){
    temp++;
  }
  return temp;
}

//
BOOL WriteTag(FILE *pszFile,char *ID,char *string)
{
  CommonChunk	common;
  DWORD		bytesRead;

  memcpy(common.chunkID,ID,4);
  common.chunkSize = strlen(string)+1;
  if(memcmp(ID,"ILYC",4) == 0){
    if(common.chunkSize > 6){
      if(memcmp(string,"LYRICS",6) == 0){
	::WriteFile(pszFile,&common,sizeof(common),&bytesRead,NULL);
	::WriteFile(pszFile,string,common.chunkSize-1,&bytesRead,NULL);
	return false;
      }
    }
  }
  ::WriteFile(pszFile,&common,sizeof(common),&bytesRead,NULL);
  if((common.chunkSize%2) == 1){
    ::WriteFile(pszFile,strcat(string,"\0\0"),common.chunkSize+1,&bytesRead,NULL);
  }else{
    ::WriteFile(pszFile,string,common.chunkSize,&bytesRead,NULL);
  }

  return true;
}
